@@ -6,6 +6,7 @@ import datetime
 import subprocess
 import platform
 from minio import Minio
+import zipfile
 
 from kabaret import flow
 from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
@@ -255,6 +256,11 @@ class MinioDownloadFile(flow.Object):
             local_path
         )
 
+        if os.path.splitext(local_path)[1] == ".zip":
+            # Unzip
+            with zipfile.ZipFile(local_path, 'r') as zip_wc:
+                zip_wc.extractall(os.path.dirname(local_path))
+
 
 class Synchronize(ProcessJobs):
 
@@ -355,7 +361,7 @@ class Synchronize(ProcessJobs):
         self.root().project().synchronization.touch()
 
 
-class SiteChoiceValue(flow.values.ChoiceValue):
+class RequestedSiteChoiceValue(flow.values.ChoiceValue):
     _revision = flow.Parent(2)
 
     def choices(self):
@@ -368,17 +374,31 @@ class SiteChoiceValue(flow.values.ChoiceValue):
         return site_names
 
 
-class Request(flow.Action):
+class RequestingSiteChoiceValue(flow.values.ChoiceValue):
+    _revision = flow.Parent(2)
+
+    def choices(self):
+        site_names = []
+        
+        for site in self._revision.sync.mapped_items():
+            if site.status.get() == "NotAvailable":
+                site_names.append(site.name())
+        
+        return site_names
+
+
+class RequestAs(flow.Action):
 
     _revision = flow.Parent()
     priority = flow.IntParam(50)
-    site_name = flow.Param(None, SiteChoiceValue).ui(label="Site to query")
+    requesting_site_name = flow.Param(None, RequestingSiteChoiceValue).ui(label="Requesting site")
+    requested_site_name = flow.Param(None, RequestedSiteChoiceValue).ui(label="Site to query")
 
     def get_buttons(self):
-        site_names = self.site_name.choices()
+        site_names = self.requested_site_name.choices()
         # Actually, an existing revision implies there is at least one
         # site on which it is available, which is the source site
-        self.site_name.set(site_names[0])
+        self.requested_site_name.set(site_names[0])
 
         return ["Request", "Cancel"]
     
@@ -386,31 +406,30 @@ class Request(flow.Action):
         return (
             context
             and not self._revision.is_working_copy()
-            and not self._revision.get_sync_status() == "Available"
+            and self.root().project().get_current_site().request_files_from_anywhere.get()
         )
 
     def run(self, button):
         if button == "Cancel":
             return
+        
+        requesting_site_name = self.requesting_site_name.get()
 
-        # Check if the version is not already present
-        current_site = self.root().project().get_current_site()
-        current_site_status = self._revision.sync[current_site.name()]
-
-        # Get requested site
+        # Get requesting and requested sites
         sites = self.root().project().admin.sites
-        requested_site = sites[self.site_name.get()]
+        requested_site = sites[self.requested_site_name.get()]
+        requesting_site = sites[requesting_site_name]
         
         # Add a download job for the requesting site
-        current_site.queue.submit_job(
+        requesting_site.queue.submit_job(
             job_type="Download",
             init_status="WAITING",
             emitter_oid=self._revision.oid(),
             user=self.root().project().get_user(),
-            studio=current_site_status.name(),
+            studio=requesting_site_name,
             priority=self.priority.get(),
         )
-        self._revision.set_sync_status("Requested")
+        self._revision.set_sync_status("Requested", site_name=requesting_site_name)
         self._revision._revisions.touch()
         
         # Check if the version is not available on the exchange server
@@ -426,9 +445,9 @@ class Request(flow.Action):
                 return self.get_result()
             
         # Check if the source version upload is not already requested
-        for job in requested_site.queue.mapped_items():
-            if job.emitter_oid.get() == self._revision.oid() and job.type.get() == "Upload":
-                self.root().session.log_warning(
+        for job in requested_site.queue.jobs(type="Upload", status="WAITING"):
+            if job.emitter_oid.get() == self._revision.oid():
+                self.root().session().log_warning(
                     "Revision already requested for upload in source site"
                 )
                 self.root().project().synchronization.touch()
@@ -440,7 +459,7 @@ class Request(flow.Action):
             init_status="WAITING",
             emitter_oid=self._revision.oid(),
             user=self.root().project().get_user(),
-            studio=current_site_status.name(),
+            studio=requesting_site_name,
             priority=self.priority.get(),
         )
 
@@ -448,6 +467,27 @@ class Request(flow.Action):
         self.root().project().synchronization.touch()
 
 
+class Request(RequestAs):
+
+    _revision = flow.Parent()
+    priority = flow.IntParam(50)
+    requesting_site_name = flow.Param("").ui(hidden=True)
+
+    def get_buttons(self):
+        self.requesting_site_name.set(
+            self.root().project().admin.current_site_name.get()
+        )
+
+        return super(Request, self).get_buttons()
+    
+    def allow_context(self, context):
+        return (
+            context
+            and not self._revision.is_working_copy()
+            and not self._revision.get_sync_status() == "Available"
+        )
+
+
 class ResetJobStatuses(flow.Action):
     _site = flow.Parent()
     status = flow.Param("WAITING", JobStatus)
@@ -473,6 +513,7 @@ class ClearQueue(flow.Action):
 class Site(flow.Object):
     short_name = flow.Param("")
     description = flow.Param("")
+    request_files_from_anywhere = flow.BoolParam(False).ui(tooltip="Allowed to request files for any site")
 
     site_type = flow.Param("Studio", StaticSiteTypeChoices)
 
